package com.example.autoservice.cars.api;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.modelmapper.ModelMapper;
import org.modelmapper.TypeToken;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.example.autoservice.cars.model.CarsEntity;
import com.example.autoservice.cars.model.CarsGrouped;
import com.example.autoservice.cars.service.CarsService;
import com.example.autoservice.clients.api.ClientsDto;
import com.example.autoservice.clients.model.ClientsEntity;
import com.example.autoservice.core.configuration.Constants;

import jakarta.validation.Valid;

@Controller
@RequestMapping(CarsController.URL)
public class CarsController {
    public static final String URL = "/cars";
    private static final String CARS_VIEW = "cars";
    private static final String CARS_ATTRIBUTE = "cars";
    private static final String CLIENTS_ATTRIBUTE = "clients";
    private static final String CARS_EDIT_VIEW = "car-edit";
    private final CarsService carsService;
    private final ModelMapper modelMapper;

    public CarsController(CarsService carsService, ModelMapper modelMapper) {
        this.carsService = carsService;
        this.modelMapper = modelMapper;
    }

    private CarsDto toDto(CarsEntity entity) {
        return modelMapper.map(entity, CarsDto.class);
    }

    private CarsGroupedDto toGroupedDto(CarsGrouped entity){
        return modelMapper.map(entity, CarsGroupedDto.class);
    }

    private CarsEntity toEntity(CarsDto dto) {
        return modelMapper.map(dto, CarsEntity.class);
    }
    
    private CarsEntity toEntity(CarsGroupedDto dto){
        return modelMapper.map(dto, CarsEntity.class);
    }

    private List<ClientsDto> getClients(){
       return modelMapper.map(carsService.getAllClients(), new TypeToken<List<ClientsDto>>(){}.getType());
    }

    @GetMapping
    public String getAll(Model model){
        List<CarsGroupedDto> cars = carsService.getAllCarsWithClients().stream().map(this::toGroupedDto).toList();
        model.addAttribute("cars",
            cars);
        return CARS_VIEW;
    }

    @GetMapping("/edit/")
    public String create(Model model){
        model.addAttribute(CARS_ATTRIBUTE, new CarsGroupedDto());
        model.addAttribute(CLIENTS_ATTRIBUTE, getClients());
        return CARS_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(
            @ModelAttribute(name = CARS_ATTRIBUTE) @Valid CarsGroupedDto car,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return CARS_EDIT_VIEW;
        }
        carsService.create(toEntity(car));
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute(CARS_ATTRIBUTE, toGroupedDto(carsService.getCarWithClient(id)));
        model.addAttribute(CLIENTS_ATTRIBUTE, getClients());
        return CARS_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @ModelAttribute(name = CARS_ATTRIBUTE) @Valid CarsGroupedDto car,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return CARS_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        carsService.update(id, toEntity(car));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id) {
        carsService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}