package com.example.autoservice.cars.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.autoservice.cars.model.CarsEntity;
import com.example.autoservice.cars.model.CarsGrouped;
import com.example.autoservice.cars.repository.CarsRepository;
import com.example.autoservice.clients.model.ClientsEntity;
import com.example.autoservice.core.error.NotFoundException;

@Service
public class CarsService {
    private final CarsRepository repository;

    public CarsService(CarsRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<CarsEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public CarsEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(CarsEntity.class, id));
    }

    @Transactional
    public CarsEntity create(CarsEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    @Transactional
    public CarsEntity update(Long id, CarsEntity entity) {
        final CarsEntity existsEntity = get(id);
        existsEntity.setMark(entity.getMark());
        existsEntity.setModel(entity.getModel());
        existsEntity.setSerial_Number(entity.getSerial_Number());
        existsEntity.setClient(entity.getClient());
        return repository.save(existsEntity);
    }

    @Transactional
    public CarsEntity delete(Long id) {
        final CarsEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
    
    @Transactional
    public List<CarsGrouped> getAllCarsWithClients(){
        return repository.findAllCarsWithClients();
    }

    @Transactional
    public CarsGrouped getCarWithClient(Long carId){
        return repository.findCarWithClient(carId);
    }

    @Transactional
    public List<ClientsEntity> getAllClients(){
        return repository.findAllClients();
    }
}
