package com.example.autoservice.clients.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.example.autoservice.clients.service.ClientsService;
import com.example.autoservice.core.configuration.Constants;
import com.example.autoservice.clients.model.ClientsEntity;

import jakarta.validation.Valid;

@Controller
@RequestMapping(ClientsController.URL)
public class ClientsController {
    public static final String URL = "/clients";
    private static final String CLIENTS_VIEW = "clients";
    private static final String CLIENTS_ATTRIBUTE = "clients";
    private static final String CLIENTS_EDIT_VIEW = "client-edit";
    private final ClientsService clientsService;
    private final ModelMapper modelMapper;

    public ClientsController(ClientsService clientsService, ModelMapper modelMapper) {
        this.clientsService = clientsService;
        this.modelMapper = modelMapper;
    }

    private ClientsDto toDto(ClientsEntity entity) {
        return modelMapper.map(entity, ClientsDto.class);
    }

    private ClientsEntity toEntity(ClientsDto dto) {
        return modelMapper.map(dto, ClientsEntity.class);
    }
    @GetMapping
    public String getAll(Model model){
        List<ClientsDto> lines = clientsService.getAll().stream()
        .map(this::toDto)
        .toList();
        model.addAttribute("clients",
            lines);
        return CLIENTS_VIEW;
    }

    @GetMapping("/edit/")
    public String create(Model model){
        model.addAttribute(CLIENTS_ATTRIBUTE, new ClientsDto());
        return CLIENTS_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(
            @ModelAttribute(name = CLIENTS_ATTRIBUTE) @Valid ClientsDto client,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return CLIENTS_EDIT_VIEW;
        }
        clientsService.create(toEntity(client));
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute(CLIENTS_ATTRIBUTE, toDto(clientsService.get(id)));
        return CLIENTS_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @ModelAttribute(name = CLIENTS_ATTRIBUTE) @Valid ClientsDto client,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return CLIENTS_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        clientsService.update(id, toEntity(client));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id) {
        clientsService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}
