package com.example.autoservice.clients.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.autoservice.clients.model.ClientsEntity;
import com.example.autoservice.clients.repository.ClientsRepository;
import com.example.autoservice.core.error.NotFoundException;

@Service
public class ClientsService {
    private final ClientsRepository repository;

    public ClientsService(ClientsRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<ClientsEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public ClientsEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(ClientsEntity.class, id));
    }

    @Transactional
    public ClientsEntity create(ClientsEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    @Transactional
    public ClientsEntity update(Long id, ClientsEntity entity) {
        final ClientsEntity existsEntity = get(id);
        existsEntity.setFirst_Name(entity.getFirst_Name());
        existsEntity.setLast_Name(entity.getLast_Name());
        existsEntity.setMiddle_Name(entity.getMiddle_Name());
        existsEntity.setDate_Birthday(entity.getDate_Birthday());
        existsEntity.setPhone_Number(entity.getPhone_Number());
        return repository.save(existsEntity);
    }

    @Transactional
    public ClientsEntity delete(Long id) {
        final ClientsEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
